#include	"ksysdefs.h"
#include	"ksysvtyp.h"
#include	"ksysprot.h"
#include	"ksyskwik.h"
/**/
/* This is a simplified version of the pathalias hashing function.
 * Thanks to Steve Belovin and Peter Honeyman
 *
 * hash a string into a long int.  31 bit crc (from andrew appel).
 * the crc table is computed at run time by crcinit() -- we could
 * precompute, but it takes 1 clock tick on a 750.
 *
 * This fast table calculation works only if POLY is a prime polynomial
 * in the field of integers modulo 2.  Since the coefficients of a
 * 32-bit polynomial won't fit in a 32-bit word, the high-order bit is
 * implicit.  IT MUST ALSO BE THE CASE that the coefficients of orders
 * 31 down to 25 are zero.  Happily, we have candidates, from
 * E. J.  Watson, "Primitive Polynomials (Mod 2)", Math. Comp. 16 (1962):
 *	x^32 + x^7 + x^5 + x^3 + x^2 + x^1 + x^0
 *	x^31 + x^3 + x^0
 *
 * We reverse the bits to get:
 *	111101010000000000000000000000001 but drop the last 1
 *         f   5   0   0   0   0   0   0
 *	010010000000000000000000000000001 ditto, for 31-bit crc
 *	   4   8   0   0   0   0   0   0
 */

#define POLY 0x48000000L	/* 31-bit polynomial (avoids sign problems) */

static long CrcTable[128];

static void
crcinit()
{	register int i, j;
	register long sum;

	for (i = 0; i < 128; ++i) {
		sum = 0L;
		for (j = 7 - 1; j >= 0; --j)
			if (i & (1 << j))
				sum ^= POLY >> j;
		CrcTable[i] = sum;
	}
	return;
}
NOSIGN
strhash(is)
char	*is;
{
	register        char    *isx = is;
	register        long    hv = 0L;
	static			int		setup = 0;
	if(!setup)	{
		crcinit();
		setup += 1;
		}
	while(*isx != ZERO)
	if(islower(*isx))       {
		hv = (hv >> 7) ^ CrcTable[(hv ^ (*isx++)) & 0x7f];
		}
	else
	if(isupper(*isx))       {
		*isx = tolower(*isx);
		hv = (hv >> 7) ^ CrcTable[(hv ^ (*isx++)) & 0x7f];
		}
	else
	if(isdigit(*isx))       {
		hv = (hv >> 7) ^ CrcTable[(hv ^ (*isx++)) & 0x7f];
		}
	else    ++isx;
	return((NOSIGN) (hv % modfac));
}
long
setmod(mf)
long	mf;
{
	auto	long	pf = modfac;
	if(mf == 0)	mf = MODFAC;
	if(kenv.ndexed && mf != pf)	{
		errmsg("<save or remove current index>\n");
		terror(FUBAR);
		}
	modfac = mf;
	return(pf);
}
